README = {
    "objects": {
        "geo-watch-Cache-Creek-2024-10-06-a": [],
        "geo-watch-Cache-Creek-2x-wider-2024-10-06-a": [],
        "geo-watch-Cache-Creek-2024-11-05": [],
        "geo-watch-Cache-Creek-2x-wider-2024-11-05": [],
    },
    "thumbnail": {
        "scale": 4,
    },
}



from typing import Dict
import requests
import geopandas as gpd
import pandas as pd
from shapely.geometry import Point

from blueness import module
from blue_objects import host
from blue_objects import metadata, file, objects
from blue_geo import NAME
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)

name = "global-power-plant-database"

template_name = f"{name}-template-v1"


url: Dict[str, str] = {
    "dataset": "https://datasets.wri.org/datasets/global-power-plant-database",
    "research": "https://www.wri.org/research/global-database-power-plants",
    "metadata": "https://datasets.wri.org/api/3/action/resource_show?id=66bcdacc-3d0e-46ad-9271-a5a76b1853d2",
}

version = "v2"


def ingest(
    object_name: str,
    version: str = "v1",
    overwrite: bool = False,
    log: bool = True,
    verbose: bool = False,
) -> bool:
    full_object_name = f"{object_name}-{version}"

    logger.info(
        "{}.ingest({}) {}".format(
            NAME,
            full_object_name,
            ",".join((["overwrite"] if overwrite else [])),
        )
    )

    response = requests.get(url["metadata"])
    # https://chat.openai.com/c/6deb94d0-826a-48de-b5ef-f7d8da416c82
    # response.raise_for_status()
    if response.status_code // 100 != 2:
        logger.error("failed to access metadata.")
        return False
    if verbose:
        logger.info(response.json())

    download_url = response.json().get("result", {}).get("url", "")
    if not download_url:
        logger.error("failed to get download_url.")
        return False

    zip_filename = objects.path_of(
        download_url.split("/")[-1],
        full_object_name,
        create=True,
    )
    if not file.download(
        url=download_url,
        filename=zip_filename,
        log=log,
        overwrite=overwrite,
    ):
        return False

    if not host.unzip(zip_filename, log=log):
        return False

    success, df = file.load_dataframe(
        objects.path_of(
            "global_power_plant_database.csv",
            full_object_name,
        ),
        log=log,
    )
    if not success:
        return False

    df["latitude"] = pd.to_numeric(df["latitude"], errors="coerce")
    df["longitude"] = pd.to_numeric(df["longitude"], errors="coerce")

    df = df.dropna(subset=["latitude", "longitude"])

    df["geometry"] = df.apply(
        lambda row: Point(row["longitude"], row["latitude"]),
        axis=1,
    )

    gdf = gpd.GeoDataFrame(
        df,
        geometry="geometry",
        crs="EPSG:4326",
    )

    if not file.save_geojson(
        objects.path_of(
            "global_power_plant_database.geojson",
            full_object_name,
        ),
        gdf,
        log=log,
    ):
        return False

    return metadata.post_to_object(
        full_object_name,
        "ingest",
        {
            "api-response": response.json(),
        },
    )



import os

from blue_objects import file, README
from blue_objects.env import ABCLI_PUBLIC_PREFIX

from blue_geo import NAME, VERSION, ICON, REPO_NAME
from blue_geo.objects import special_objects


def build() -> bool:
    return all(
        README.build(
            path=os.path.join(
                file.path(__file__),
                f"md/{suffix}",
            ),
            macros=macros,
            ICON=ICON,
            NAME=NAME,
            VERSION=VERSION,
            REPO_NAME=REPO_NAME,
        )
        for suffix, macros, in [
            (
                f"{object_name}.md".replace("-", "_"),
                {
                    "--object-name--": [
                        "💾 [{}-{}]({}/{}-{}.tar.gz)".format(
                            object_name,
                            special_objects[object_name].version,
                            ABCLI_PUBLIC_PREFIX,
                            object_name,
                            special_objects[object_name].version,
                        ),
                    ],
                    "--urls--": sorted(
                        [
                            f" - {title}: {url}"
                            for title, url in special_objects[object_name].url.items()
                        ]
                    ),
                },
            )
            for object_name in [
                "global-power-plant-database",
            ]
        ]
    )



from typing import List

from blue_options.terminal import show_usage


def help_download(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "QGIS",
            "templates",
            "download",
        ],
        "download QGIS templates.",
        {
            "from: $BLUE_GEO_QGIS_TEMPLATES_OBJECT_NAME": "",
            "to: $BLUE_GEO_QGIS_PATH_TEMPLATES": "",
        },
        mono=mono,
    )


def help_upload(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "QGIS",
            "templates",
            "upload",
        ],
        "upload QGIS templates.",
        {
            "from: $BLUE_GEO_QGIS_PATH_TEMPLATES": "",
            "to: $BLUE_GEO_QGIS_TEMPLATES_OBJECT_NAME": "",
        },
        mono=mono,
    )


help_functions = {
    "download": help_download,
    "upload": help_upload,
}



from typing import List

from blue_options.terminal import show_usage, xtra

from blue_geo.objects import special_objects


def help_ingest(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            "publish,upload",
            xtra(",version=<v1>", mono=mono),
        ]
    )

    args = [
        "[--overwrite 1]",
    ]

    return show_usage(
        [
            "@geo",
            "ingest",
            f"[{options}]",
            "<object-name>",
        ]
        + args,
        "ingest <object-name>.",
        {
            "object-name: {}".format(
                " | ".join(
                    "{}[-{}]".format(key, object_module.version)
                    for key, object_module in special_objects.items()
                )
            ): [],
        },
        mono=mono,
    )



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from blue_geo import NAME
from blue_geo.env import BLUE_GEO_WATCH_ALGO_DIFF_MAP_DYNAMIC_RANGE
from blue_geo.watch.algo.diff.map import map_function
from blue_geo.watch.algo.modality.reduce import reduce_function
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)

list_of_tasks = "map|reduce"


parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help=list_of_tasks,
)
parser.add_argument(
    "--offset",
    type=str,
    default="000",
)
parser.add_argument(
    "--object_name",
    type=str,
)
parser.add_argument(
    "--query_object_name",
    type=str,
)
parser.add_argument(
    "--suffix",
    type=str,
)
parser.add_argument(
    "--depth",
    type=int,
    default=2,
)
parser.add_argument(
    "--dynamic_range",
    type=float,
    default=float(BLUE_GEO_WATCH_ALGO_DIFF_MAP_DYNAMIC_RANGE),
)
args = parser.parse_args()

success = args.task in list_of_tasks
if args.task == "map":
    success = map_function(
        query_object_name=args.query_object_name,
        suffix=args.suffix,
        offset=args.offset,
        depth=args.depth,
        dynamic_range=args.dynamic_range,
    )
elif args.task == "reduce":
    success = reduce_function(
        query_object_name=args.query_object_name,
        suffix=args.suffix,
        object_name=args.object_name,
        content_threshold=-1,
        list_of_suffix=["histogram"],
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from typing import Dict
from types import ModuleType


from blue_geo.objects import global_power_plant_database

special_objects: Dict[str, ModuleType] = {
    "global-power-plant-database": global_power_plant_database,
}



# to avoid import complications.
# from blue_objects.env import ABCLI_PUBLIC_PREFIX
ABCLI_PUBLIC_PREFIX = "https://kamangir-public.s3.ca-central-1.amazonaws.com"


NAME = "blue_geo"

ICON = "🌐"

DESCRIPTION = f"{ICON} AI for a Blue Planet."

VERSION = "4.833.1"

REPO_NAME = "blue-geo"

MARQUEE = f"{ABCLI_PUBLIC_PREFIX}/2024-01-06-20-39-46-73614/2024-01-06-20-39-46-73614-2X.gif?raw=true"

ALIAS = "@geo"


def fullname() -> str:
    return f"{NAME}-{VERSION}"



from typing import List

from blue_options.terminal import show_usage, xtra
from blueflow.workflow.runners import list_of_runners

from blue_geo.datacube.modalities import options as modality_options
from blue_geo.watch.targets.target_list import TargetList
from blue_geo.watch.algo import list_of_algo
from blue_geo.help.watch.targets import help_functions as help_targets


def help_(
    tokens: List[str],
    mono: bool,
    help_batch: bool = False,
) -> str:
    options = "".join(
        [
            "batch",
            xtra(",dryrun,name=<job-name>", mono),
        ]
    )

    target_list = TargetList()

    target_options = "".join(
        [
            xtra("<query-object-name> | ", mono),
            "target=<target>",
        ]
    )

    algo_options = "algo=<algo>,<algo-options>"

    workflow_options = "".join(
        [
            xtra("dryrun,~submit,", mono),
            "to=<runner>",
        ]
    )

    map_options = "".join(
        [
            xtra("dryrun,<map-options>", mono),
        ]
    )

    reduce_options = "".join(
        [
            xtra("dryrun,~gif,", mono),
            "publish",
            xtra(",<reduce-options>", mono=mono),
        ]
    )

    return show_usage(
        [
            "@geo",
            "watch",
            options if help_batch else f"[{options}]",
            f"[{target_options}]",
            f"[{algo_options}]",
            f"[{workflow_options}]",
            f"[{map_options}]",
            f"[{reduce_options}]",
            "[-|<object-name>]",
        ],
        "watch target -{}> <object-name>.".format("aws-batch-" if help_batch else ""),
        {
            "algo: diff | modality": [],
            "<algo-options>:": [
                "diff: modality=<modality>,range=<100.0>",
                "modality: modality=<modality>",
            ],
            "modality: {}".format(" | ".join(modality_options)): [],
            "runner: {}".format(" | ".join(list_of_runners())): [],
            "target: {}".format(" | ".join(target_list.get_list())): [],
        },
        mono=mono,
    )


def help_batch(
    tokens: List[str],
    mono: bool,
) -> str:
    return help_(
        tokens=tokens,
        mono=mono,
        help_batch=True,
    )


def help_map(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            "algo=<algo>",
            xtra(",dryrun,~download,", mono),
            "modality=<modality>,",
            "offset=<offset>,suffix=<suffix>",
            xtra(",~upload", mono),
        ]
    )

    return show_usage(
        [
            "@geo",
            "watch",
            "map",
            f"[{options}]",
            "[.|<query-object-name>]",
        ],
        "@geo watch map <query-object-name> @ <offset> -> /<suffix>.",
        mono=mono,
    )


def help_query(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            xtra("dryrun,", mono=mono),
            "target=<target>",
            xtra(",~upload", mono=mono),
        ]
    )

    return show_usage(
        [
            "@geo",
            "watch",
            "query",
            f"[{options}]",
            "[.|<object-name>]",
        ],
        "query target -> <object-name>.",
        mono=mono,
    )


def help_reduce(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            "algo=<algo>",
            xtra("dryrun,~download,", mono),
            "publish,suffix=<suffix>",
            xtra(",~upload", mono),
        ]
    )

    return show_usage(
        [
            "@geo",
            "watch",
            "reduce",
            f"[{options}]",
            "[..|<query-object-name>]",
            "[.|<object-name>]",
        ],
        "@geo watch reduce <query-object-name>/<suffix> -> <object-name>.",
        mono=mono,
    )


help_functions = {
    "": help_,
    "batch": help_batch,
    "map": help_map,
    "query": help_query,
    "reduce": help_reduce,
    "targets": help_targets,
}



import os
from qgis.core import *
from qgis.gui import *

# version 2.4.1


@qgsfunction(args="auto", group="Custom", referenced_columns=[])
def vanwatch_temporal(layer_name, project_path, feature, parent):
    """
    Calculates the sum of the two parameters value1 and value2.
    <h2>Example usage:</h2>
    <ul>
      <li>my_sum(5, 8) -> 13</li>
      <li>my_sum("field1", "field2") -> 42</li>
    </ul>
    """
    list_of_layers = []
    for _, _, files in os.walk(os.path.dirname(project_path)):
        for filename in files:
            if filename.endswith(".geojson"):
                list_of_layers += [filename.split(".")[0]]

    list_of_layers = sorted(list_of_layers)

    if layer_name not in list_of_layers:
        return "x1"

    index = list_of_layers.index(layer_name)

    return list_of_layers[index + 1] if index < len(list_of_layers) - 1 else "x2"



from typing import Dict
from tqdm import trange
import numpy as np
import cv2

from blueness import module
from blue_options import string
from blue_objects.logger.matrix import log_matrix, log_matrix_hist
from blue_objects import file, objects
from blue_objects.metadata import post_to_object
from blueflow import fullname as blueflow_fullname

from blue_geo import fullname
from blue_geo import NAME
from blue_geo.env import BLUE_GEO_WATCH_ALGO_DIFF_MAP_DYNAMIC_RANGE
from blue_geo.catalog.generic import GenericDatacube
from blue_geo.watch.workflow.common import load_watch
from blue_geo.logger import logger


NAME = module.name(__file__, NAME)


def map_function(
    query_object_name: str,
    suffix: str,
    offset: str,
    depth: int,
    dynamic_range: float = float(BLUE_GEO_WATCH_ALGO_DIFF_MAP_DYNAMIC_RANGE),
    line_width: int = 80,
    colorbar_width: int = 20,
    min_width: int = 1200,
) -> bool:
    if depth < 2:
        logger.error(f"depth={depth} < 2!")
        return False

    offset_int = int(offset)

    object_name = f"{query_object_name}-{suffix}-{offset}"

    success, target, _ = load_watch(object_name, log=False)
    if not success:
        return success

    logger.info(
        "{}.map: {} #{}+D={} -> {}".format(
            NAME,
            target.one_liner,
            offset,
            depth,
            object_name,
        )
    )

    acquisition_metadata: Dict[str, Dict] = {}
    baseline_filename: str = ""
    target_filename: str = ""
    for index in trange(depth):
        logger.info(f"processing metadata: index={index} ...")

        index_object_name = "{}-D-{:03d}".format(
            object_name,
            offset_int + index,
        )
        success, acquisition_metadata[index] = file.load_yaml(
            objects.path_of(
                filename="metadata.yaml",
                object_name=index_object_name,
            ),
            ignore_error=True,
        )
        if not success:
            break

        if index in [0, 1]:
            filename = objects.path_of(
                filename=acquisition_metadata[index].get("map", {}).get("filename", ""),
                object_name=index_object_name,
            )
            if not file.exists(filename):
                logger.warning(f"{filename} not found!")
                success = False
                break

            if index == 0:
                baseline_filename = filename
            else:
                target_filename = filename

            if not file.copy(
                filename,
                objects.path_of(
                    file.name_and_extension(filename),
                    object_name,
                ),
            ):
                success = False
                break

    baseline_metadata = {}
    if success:
        success, baseline_image, baseline_metadata = GenericDatacube.load_geoimage(
            filename=baseline_filename,
            log=True,
        )

    if success:
        success, target_image, _ = GenericDatacube.load_geoimage(
            filename=target_filename,
            log=True,
        )

    if success:
        diff_image = np.squeeze(
            target_image.astype(np.float32) - baseline_image.astype(np.float32)
        )

        diff_image_pretty_shape = string.pretty_shape_of_matrix(diff_image)

        success = file.save_matrix(
            objects.path_of(
                "{}-diff.npy".format(file.name(target_filename)),
                object_name,
            ),
            diff_image,
        )

    if success:
        log_matrix_hist(
            matrix=diff_image,
            dynamic_range=(-dynamic_range, dynamic_range),
            header=[
                "diff histogram",
                query_object_name,
                f"/{suffix}",
                f"offset: {offset}",
                f"depth: {depth}",
                file.name_and_extension(baseline_filename),
                file.name_and_extension(target_filename),
                diff_image_pretty_shape,
                "pixel_size: {} m".format(baseline_metadata.get("pixel_size", -1.0)),
            ],
            footer=["DN diff"],
            filename=objects.path_of(
                "{}-diff-histogram.png".format(file.name(target_filename)),
                object_name,
            ),
            line_width=line_width,
        )

    diff_filename = objects.path_of(
        "{}-diff.png".format(file.name(target_filename)),
        object_name,
    )
    if success:
        if not log_matrix(
            matrix=diff_image[:, :, 0],
            header=[
                " | ".join(
                    objects.signature(
                        " | ".join(
                            [
                                suffix,
                                f"offset: {offset}",
                                f"depth: {depth}",
                                file.name_and_extension(baseline_filename),
                                file.name_and_extension(target_filename),
                                diff_image_pretty_shape,
                                "pixel_size: {} m".format(
                                    baseline_metadata.get("pixel_size", -1.0)
                                ),
                            ]
                        ),
                        query_object_name,
                    )
                ),
            ],
            footer=[
                target.one_liner,
                fullname(),
                blueflow_fullname(),
            ],
            filename=diff_filename,
            dynamic_range=(-dynamic_range, dynamic_range),
            line_width=line_width,
            min_width=min_width,
            colorbar_width=colorbar_width,
            colormap=cv2.COLORMAP_JET,
        ):
            success = False

    if not success:
        logger.warning("not usable.")

    return post_to_object(
        object_name,
        "map",
        {
            "algo": "diff",
            "inputs": {
                "offset": offset,
                "depth": depth,
                "acquisitions": acquisition_metadata,
            },
            "filename": file.name_and_extension(diff_filename),
            "target": target.__dict__,
            "usable": success,
        },
    )



from typing import List

from blue_options.terminal import show_usage, xtra

from blue_geo.help.QGIS.expressions import help_functions as help_expressions
from blue_geo.help.QGIS.templates import help_functions as help_templates


def help_download(
    tokens: List[str],
    mono: bool,
) -> str:
    open_options = "".join(
        [
            "open",
            xtra(",~QGIS", mono=mono),
        ]
    )

    return show_usage(
        [
            "QGIS",
            "download",
            "[.|<object-name>]",
            f"[{open_options}]",
        ],
        "download object and its QGIS dependencies.",
        mono=mono,
    )


def help_seed(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "QGIS",
            "seed",
            "[screen]",
        ],
        "seed 🌱 QGIS.",
        mono=mono,
    )


def help_server(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "QGIS",
            "server",
            "[start]",
        ],
        "start QGIS server.",
        mono=mono,
    )


def help_upload(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "QGIS",
            "upload",
            "[.|<object-name>]",
        ],
        "upload object and its QGIS dependencies.",
        mono=mono,
    )


help_functions = {
    "download": help_download,
    "expressions": help_expressions,
    "seed": help_seed,
    "server": help_server,
    "templates": help_templates,
    "upload": help_upload,
}



import numpy as np

from blueness import module
from blue_objects import file, objects
from blue_objects.logger.matrix import log_matrix
from blue_objects.metadata import post_to_object, get_from_object
from blueflow import fullname as blueflow_fullname

from blue_geo import fullname
from blue_geo import NAME
from blue_geo.catalog.functions import get_datacube_class
from blue_geo.watch.workflow.common import load_watch
from blue_geo.logger import logger


NAME = module.name(__file__, NAME)


def map_function(
    query_object_name: str,
    suffix: str,
    offset: str,
    modality: str,
    line_width: int = 80,
    min_width: int = 1200,
) -> bool:
    offset_int = int(offset)

    object_name = f"{query_object_name}-{suffix}-{offset}"

    list_of_datacube_id = get_from_object(
        query_object_name,
        "datacube_id",
        [],
    )
    if len(list_of_datacube_id) < offset_int + 1:
        logger.warning(f"offset={offset}: datacube-id not found.")
        return True
    datacube_id = list_of_datacube_id[offset_int]

    success, target, list_of_files = load_watch(object_name)
    if not success or not list_of_files:
        return success
    filename = list_of_files[0]

    logger.info(
        "{}.map: {} #{} @ {} -{}-> {}".format(
            NAME,
            target.one_liner,
            offset,
            datacube_id,
            modality,
            object_name,
        )
    )

    datacube_class = get_datacube_class(datacube_id)
    success, frame, frame_file_metadata = datacube_class.load_geoimage(
        filename,
        log=True,
    )

    if not file.save_matrix(
        file.add_extension(filename, "npy"),
        frame,
    ):
        success = False

    if frame.dtype == np.uint16:
        frame = frame.astype(np.float32) / 5000 * 255
        frame[frame < 0] = 0
        frame[frame > 255] = 255
        frame = frame.astype(np.uint8)

    frame_fp32 = frame.astype(np.float32).flatten() / 255
    frame_fp32 = frame_fp32[frame_fp32 > 0.0]
    frame_fp32 = frame_fp32[frame_fp32 < 1.0]
    content_ratio = len(frame_fp32) / frame.size

    if not log_matrix(
        matrix=frame,
        header=[
            " | ".join(
                objects.signature(
                    "{} / {}".format(
                        datacube_id,
                        file.name_and_extension(filename),
                    ),
                    object_name,
                )
                + [
                    "pixel_size: {} m".format(
                        frame_file_metadata.get("pixel_size", -1.0)
                    ),
                    "content: {:05.1f}%".format(content_ratio * 100.0),
                    f"#{offset}",
                ]
            ),
        ],
        footer=[
            target.one_liner,
            fullname(),
            blueflow_fullname(),
        ],
        filename=file.add_extension(filename, "png"),
        line_width=line_width,
        min_width=min_width,
    ):
        success = False

    return post_to_object(
        object_name,
        "map",
        {
            "algo": "modality",
            "inputs": {
                "offset": offset,
                "modality": modality,
            },
            "content_ratio": content_ratio,
            "datacube_id": datacube_id,
            "filename": file.name_and_extension(filename),
            "target": target.__dict__,
            "usable": success,
        },
    )



from typing import List, Dict
import os

from blue_options import string
from blue_objects import file
from blue_geo.watch.targets.target_list import TargetList
from blue_geo.watch.targets.bellingcat_2024_09_27_nagorno_karabakh import (
    README as bellingcat_2024_09_27_nagorno_karabakh,
)
from blue_geo.watch.targets.burning_man_2024 import README as burning_man_2024
from blue_geo.watch.targets.Cache_Creek import README as Cache_Creek
from blue_geo.watch.targets.chilcotin_river_landslide import (
    README as chilcotin_river_landslide,
)
from blue_geo.watch.targets.DrugSuperLab import README as DrugSuperLab
from blue_geo.watch.targets.elkhema import README as elkhema
from blue_geo.watch.targets.jasper import README as jasper
from blue_geo.watch.targets.Fagradalsfjall import README as Fagradalsfjall
from blue_geo.watch.targets.Leonardo import README as Leonardo
from blue_geo.watch.targets.Mount_Etna import README as Mount_Etna
from blue_geo.watch.targets.Silver_Peak import README as Silver_Peak
from blue_objects.env import ABCLI_PUBLIC_PREFIX


list_of_targets = {
    "DrugSuperLab": DrugSuperLab,
    "chilcotin-river-landslide": chilcotin_river_landslide,
    "burning-man-2024": burning_man_2024,
    "Mount-Etna": Mount_Etna,
    "Fagradalsfjall": Fagradalsfjall,
    "Jasper": jasper,
    "Leonardo": Leonardo,
    "bellingcat-2024-09-27-nagorno-karabakh": bellingcat_2024_09_27_nagorno_karabakh,
    "elkhema": elkhema,
    "Cache-Creek": Cache_Creek,
    "Silver-Peak": Silver_Peak,
}

targets_path = file.path(__file__)

items: List[str] = []
for target_name in sorted(list_of_targets.keys()):
    target_info = list_of_targets[target_name]

    list_of_objects = target_info["objects"]
    assert isinstance(list_of_objects, dict)

    target_README = f"./targets/md/{target_name}.md"

    target_title = "`{}`".format(target_info.get("title", target_name))

    items += [
        (
            f"## [{target_title}]({target_README})"
            if file.exists(os.path.join(targets_path, target_README))
            else f"## {target_title}"
        ),
    ]

    target_list = TargetList()

    if list_of_objects:
        thumbnail_info = target_info.get("thumbnail", {})
        assert isinstance(thumbnail_info, dict)

        thumbnail_index = thumbnail_info.get("index", -1)

        thumbnail_scale = thumbnail_info.get("scale", 2)
        thumbnail_scale_str = f"-{thumbnail_scale}X" if thumbnail_scale != 1 else ""

        thumbnail_object_name = list(list_of_objects.keys())[thumbnail_index]
        items += [
            "",
            "<details>",
            "<summary>🌐</summary>",
            "",
            f"![image]({ABCLI_PUBLIC_PREFIX}/{thumbnail_object_name}/{thumbnail_object_name}{thumbnail_scale_str}.gif?raw=true&random={string.random()})",
            "",
            "</details>",
            "",
        ]

    items += target_list.get(target_name).urls_as_str()

    items += [
        "- {}.".format(
            ", ".join(
                [
                    f"[`{object_name}`]({ABCLI_PUBLIC_PREFIX}/{object_name}.tar.gz)",
                    f"[gif]({ABCLI_PUBLIC_PREFIX}/{object_name}/{object_name}.gif)",
                ]
                + description
            )
        )
        for object_name, description in list_of_objects.items()
    ]

    items += [""]

object_name = "geo-watch-bellingcat-2024-09-27-nagorno-karabakh-6X-2024-10-05-b"
macros: Dict[str, str] = {
    "--scale-note--": [
        "ℹ️ suffix published gif urls with `-2X` and `-4X` for different scales. example: {}.".format(
            ", ".join(
                [
                    "[{}X]({}/{}/{}{}.gif)".format(
                        scale,
                        ABCLI_PUBLIC_PREFIX,
                        object_name,
                        object_name,
                        "" if scale == 1 else f"-{scale}X",
                    )
                    for scale in [1, 2, 4]
                ]
            )
        )
    ]
}



from typing import List, Dict

from blue_objects import host, file, host

from blue_geo.catalog.SkyFox.classes import SkyFoxCatalog
from blue_geo.catalog.generic.generic.stac import STACDatacube
from blue_geo.catalog.generic.generic.scope import DatacubeScope
from blue_geo import env
from blue_geo.logger import logger


class SkyFoxVenusDatacube(STACDatacube):
    catalog = SkyFoxCatalog()

    collection = "venus-l2a"

    name = "Venus"

    QGIS_template = env.BLUE_GEO_QGIS_TEMPLATE_DATACUBE_SKYFOX_VENUS

    band_names: Dict[str, str] = {
        7: "red",
        4: "green",
        3: "blue",
    }

    @classmethod
    def band_index(cls, band_name: str) -> int:
        return {
            band_name: band_index for band_index, band_name in cls.band_names.items()
        }.get(band_name, -1)

    @staticmethod
    def band_suffix(
        band_index: int,
        product: str = "SRE",  # SRE | FRE
    ):
        return f"{product}_B{band_index}.tif"

    def generate(
        self,
        modality: str,
        overwrite: bool = False,
    ) -> str:
        product = ""
        if modality.startswith("rgb@"):
            modality, product = modality.split("@", 1)
        if not product:
            product = "SRE"

        # TODO: clean-up in the next refactor 🤦🏽
        if modality != "rgb":
            logger.error(f"{modality}: modality is not implemented.")
            return ""

        logger.info(
            "{}.generate({} : {})".format(
                self.__class__.__name__,
                modality,
                product,
            )
        )

        list_of_colors = ["red", "green", "blue"]

        rgb_filename: str = ""
        list_of_band_files: List[str] = []
        for color in list_of_colors:
            suffix = self.band_suffix(
                band_index=self.band_index(band_name=color),
                product=product,
            )

            candidates = self.list_of_files(DatacubeScope(suffix))
            if not candidates:
                logger.error(f"cannot find {suffix}.")
                return ""

            full_filename = self.full_filename(candidates[0])
            list_of_band_files += [full_filename]

            if not rgb_filename:
                rgb_filename = full_filename.replace(suffix, f"{product}_RGB.tif")

        if file.exists(rgb_filename) and not overwrite:
            logger.info(f"✅ {rgb_filename}")
            return ""

        return " ".join(
            [
                "gdal_merge.py",
                "-separate",
                f"-o {rgb_filename}",
            ]
            + list_of_band_files
        )

    def ingest_filename(
        self,
        filename: str,
        overwrite: bool = False,
        verbose: bool = True,
    ) -> bool:
        if super().ingest_filename(filename, overwrite, verbose):
            return True

        s3_uri = ""
        for item in self.metadata["Item"].assets.values():
            if item.href.endswith(filename):
                s3_uri = item.href
                break

        if not s3_uri:
            logger.info(f"{filename}: file not found.")
            return False

        # https://registry.opendata.aws/venus-l2a-cogs/
        return host.shell(
            "aws s3 cp --no-sign-request {} {}".format(
                s3_uri,
                self.full_filename(filename),
            ),
            log=verbose,
        )

    def list_of_files(
        self,
        scope: DatacubeScope = DatacubeScope("all"),
        verbose: bool = False,
    ) -> List[str]:
        raw_datacube_id = self.raw_datacube_id()

        product: str = "FRE" if "_FRE_" in scope.contains else "SRE"

        output = scope.filter(
            [
                {
                    "filename": (value.href.split(f"{raw_datacube_id}/", 1)[1]),
                }
                for value in self.metadata["Item"].assets.values()
                if raw_datacube_id in value.href
            ],
            needed_for_rgb=lambda filename: any(
                filename.endswith(suffix)
                for suffix in [
                    self.band_suffix(
                        band_index,
                        product,
                    )
                    for band_index in [
                        self.band_index(color)
                        for color in [
                            "red",
                            "green",
                            "blue",
                        ]
                    ]
                ]
            ),
            verbose=verbose,
        )

        if scope.rgb:
            suffix = self.band_suffix(
                self.band_index("red"),
                product,
            )
            candidates = self.list_of_files(DatacubeScope(suffix))
            if candidates:
                rgb_filename = candidates[0].replace(suffix, f"{product}_RGB.tif")
                output += [rgb_filename]

        return output



from typing import List

from blue_options.terminal import show_usage


def help_pull(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "QGIS",
            "expressions",
            "pull",
        ],
        "pull QGIS expressions.",
        {
            "from: $BLUE_GEO_QGIS_PATH_EXPRESSIONS_GIT": "",
            "to: $BLUE_GEO_QGIS_PATH_EXPRESSIONS": "",
            "https://docs.qgis.org/3.34/en/docs/user_manual/expressions": "",
        },
        mono=mono,
    )


def help_push(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "QGIS",
            "expressions",
            "push",
        ],
        "push QGIS expressions.",
        {
            "from: $BLUE_GEO_QGIS_PATH_EXPRESSIONS": "",
            "to: $BLUE_GEO_QGIS_PATH_EXPRESSIONS_GIT": "",
            "https://docs.qgis.org/3.34/en/docs/user_manual/expressions": "",
        },
        mono=mono,
    )


help_functions = {
    "push": help_push,
    "pull": help_pull,
}



import time
from typing import Union, List
import os
import random

if not QGIS_is_live:
    from log import log

blue_geo_QGIS_path_server = os.path.join(
    os.getenv("HOME", ""),
    "Downloads/QGIS/server",
)

os.makedirs(blue_geo_QGIS_path_server, exist_ok=True)


def seed(command: Union[str, List[str]]):
    if isinstance(command, list):
        command = " ".join(command)

    hash_id = "{}-{:05d}".format(
        time.strftime("%Y-%m-%d-%H-%M-%S", time.localtime(time.time())),
        random.randrange(100000),
    )

    with open(
        os.path.join(
            blue_geo_QGIS_path_server,
            f"{hash_id}.command",
        ),
        "w",
    ) as f:
        f.write(command)

    log(hash_id, command, icon="🌱")
